import java.util.*;
import java.sql.*;
import java.io.*;

public class DBInterface{
    
    public Connection conn =  null;
    private Properties dbProps = null;
    private static String confFile = "db.conf";
    private PreparedStatement pstmt;
    
    private static void loadDriver() throws Exception {
        //String driver = "oracle.jdbc.driver.OracleDriver";
        String driver = "org.gjt.mm.mysql.Driver";
        // driver = "com.mysql.jdbc.Driver";
        Class.forName(driver);
    }
    
    private void loadProperties(String propFileName)
    throws IOException {
        File f = new File(propFileName);
        System.out.println("The db.conf is in : " + f.getAbsolutePath());
        FileInputStream fis = new FileInputStream(propFileName);
        dbProps = new Properties();
        dbProps.load(fis);
    }
    
    DBInterface() {
        try{
            //System.out.println("1.1");
            loadDriver();
            //System.out.println("1.2");
            loadProperties(confFile);
            //System.out.println("1.3");
            init();
        } catch (Exception e){
            e.printStackTrace(System.out);
            System.out.println("Properties could not be loaded... Please restart the server or check for database connectivity.");
        }finally{
        }
    }
    
    public Connection getConnection() throws Exception{
        // Load the required drivers
        DBInterface.loadDriver();
        int dbPort = Integer.parseInt(dbProps.getProperty("dbport"));
        String dbHost = dbProps.getProperty("dbhost");
        String dbUser = dbProps.getProperty("dbuser");
        String dbPassword = dbProps.getProperty("dbpassword");
        String dbName = dbProps.getProperty("dbname");
        String dbType = dbProps.getProperty("dbtype");
        
        String jdbcString = "jdbc:"+dbType+"://"+dbHost+"/"+dbName+"?user="+dbUser+"&password="+dbPassword;
        // Don't print the string as then passwd will be exposed
        //System.out.println(jdbcString);
        
        if(conn == null){
            conn = DriverManager.getConnection(jdbcString);
            //System.out.println("Creating a new database connection");
        }else if(conn.isClosed()){
            System.out.println("Connection was lost, May be because of time out. Reconnect it.");
            conn=DriverManager.getConnection(jdbcString);
        }
        return conn;
    }
    
    
    public void relConnection(){
        //Hmm... Do nothing...
    }
    
    public void init() {
        try {
            if(conn != null) destroy();
            System.out.print("<");
            conn = getConnection();
            //System.out.println("1.4");
            //System.out.println(getUserPassword("dheren"));
            //String query = "CREATE TABLE G6 (login char(50), subject char(50), marks INTEGER )";
            /*String query = "INSERT INTO G6 VALUES ('Dummy', 'Maths', 9 )";
            System.out.println(query);
            PreparedStatement pstmt = a.conn.prepareStatement(query);
            pstmt.executeUpdate();*/
        } catch (Exception e) {
            e.printStackTrace();
        }
    }
    
    public void destroy(){// throws Exception
        try{
            /*if(pstmt != null) {
                pstmt.clearParameters();
                pstmt.close();
            }*/
            if(conn != null) {
                conn.close();
                conn = null;
            }
            //System.out.println("Releasing database connection.");
            System.out.print(">");
        } catch (Exception e){
            e.printStackTrace(System.out);
            System.out.println("Destrying Login servlet. Could not close database connection..");
        }finally{
        }
        //System.out.println("All Resources Released");
        
    }
    
    public ResultSet executeQuery(String query) {
        try {
            if(pstmt != null) pstmt.close();
            pstmt = conn.prepareStatement(query);
            return pstmt.executeQuery();
        } catch(java.sql.SQLException sqle) {
            System.out.println("SQL Exception found : " + sqle.getLocalizedMessage());
            sqle.printStackTrace();
        }
        return null;
    }
    
    public ResultSet getSubjectRows(String condition) {
        String query = "select l.login as login, name , subjectname as Subject ,marks as" +
        " Marks from logininfo l,subjectinfo s,marks m where l.login = m.loginid AND s.subjectid = m.subjectid" +
        condition +
        " order by l.login";
        return executeQuery(query);
    }
    
    public String  getUserPassword(String loginname) {
        String password = "";
        try {
            //init();
            ResultSet rs = executeQuery("Select password from logininfo where login = '"+loginname+"'");
            if(!rs.next()) return null;
            password = rs.getString("password");
            pstmt.close();
            //rs.close();
            //destroy();
        } catch(java.sql.SQLException sqle) {
            System.out.println("SQL Exception found : " + sqle.getLocalizedMessage());
            sqle.printStackTrace();
        }
        return(password);
    }
    
    public ResultSet getStudentMarks(String loginid) {
        // init();
        String query = "select l.login as k, name as k1, subjectname as Subject ,marks as" +
        " Marks from logininfo l,subjectinfo s,marks m where l.login = m.loginid AND s.subjectid = m.subjectid" +
        (loginid.equals("all") ? "" : " AND m.loginid = " + loginid) +
        " order by l.login";
        return executeQuery(query);
    }
    
    public ResultSet getSubjectMarks(String subjectName,String filterCondition, String filterValue)  {
        //init();
        String query = "select subjectname as k, l.login as Login, name as Name, marks as Marks" +
        " from logininfo l,subjectinfo s,marks m where l.login = m.loginid AND s.subjectid = m.subjectid" +
        (subjectName.equals("all") ? "" : " AND s.subjectname = '"+subjectName + "'") +
        (filterCondition == null ? "" : " AND m.marks " + filterCondition + " " + filterValue) +
        " order by subjectname";
        //System.out.println(query);
        return executeQuery(query);
    }
    
    
    public ResultSet getUsers() {
        //init();
        return executeQuery("select name,login from logininfo order by name");
    }
    
    public ResultSet getSubjects() {
        return executeQuery("select * from subjectinfo order by subjectid");
    }
    
    public String[] getSubjectArray() {
        try {
            java.util.List s = new java.util.LinkedList(); /* subjects */
            ResultSet rs = executeQuery("select * from subjectinfo order by subjectid");
            //s = new String[rs.getRow()];
            //int i = 0;
            while(rs.next()) s.add(rs.getString(2));
            String[] s1 = (String[])s.toArray(new String[0]);
            System.out.println(s1.length);
            return s1;
        }catch(java.sql.SQLException sqle) {
            System.out.println("SQL Exception found : " + sqle.getLocalizedMessage());
            sqle.printStackTrace();
        }
        return null;
    }
    
    public String getUserName(String loginid) {
        String name = "";
        try {
            //init();
            String query = "Select name from logininfo where login = '"+loginid+"'";
            ResultSet rs = executeQuery(query);
            rs.next();
            name = rs.getString("name");
            pstmt.close();
            //destroy();
        } catch(java.sql.SQLException sqle) {
            System.out.println("SQL Exception found : " + sqle.getLocalizedMessage());
            sqle.printStackTrace();
        }
        return(name);
    }
    
    public boolean hasBeenSurveyed(String loginid) {
        try {
            //System.out.println("1");
            //init();
            //System.out.println("2");
            String query = "select * from marks where loginid = '" + loginid + "'";
            //pstmt = conn.prepareStatement(query);
            //System.out.println("3");
            //ResultSet rs = pstmt.executeQuery();
            //System.out.println("4");
            //System.out.println("\n\nRows from marks \n");
            /*while(rs.next()){
                System.out.print("-)" +rs.getString(1));
                System.out.print("\t\t");
                System.out.print(rs.getInt(2));
                System.out.print("\t\t");
                System.out.println(rs.getInt(3));
            }*/
            //            boolean ret = rs.next();
            //boolean b = rs.next();
            //destroy();
            return executeQuery(query).next();
        }catch(java.sql.SQLException sqle) {
            System.out.println("SQL Exception found : " + sqle.getLocalizedMessage());
            sqle.printStackTrace();
        }
        return false;
    }
    
    public void putMarks(String login, String subject, String marks) {
        try {
            //init();
            String query = "Select subjectid from subjectinfo where subjectname = '" + subject + "'";
            //System.out.println(query);
            //pstmt = conn.prepareStatement(query);
            ResultSet rs = executeQuery(query);
            rs.next();
            int subjectid = rs.getInt(1);
            rs.close();
            pstmt.close();
            //destroy();
            //init();
            query = "Insert into marks values ( '" +login + "', " + subjectid + ", " + marks + ")";
            //System.out.println(query);
            pstmt = conn.prepareStatement(query);
            pstmt.executeUpdate();
            //destroy();
        } catch(java.sql.SQLException sqle) {
            System.out.println("SQL Exception found : " + sqle.getLocalizedMessage());
            sqle.printStackTrace();
        }
    }
    
    public void executeUpdate(String uq) {
        try {
            System.out.println(uq);
            conn.prepareStatement(uq).executeUpdate();
        } catch(java.sql.SQLException sqle) {
            System.out.println("SQL Exception found : " + sqle.getLocalizedMessage());
            sqle.printStackTrace();
        }
    }
    
    public void putUser(String login, String password, String name) {
        try {
            String query = "Insert into logininfo values ( '" +login + "', '" + password + "', '" + name + "')";
            //System.out.println(query);
            pstmt = conn.prepareStatement(query);
            pstmt.executeUpdate();
            pstmt.close();
            //destroy();
        } catch(java.sql.SQLException sqle) {
            System.out.println("SQL Exception found : " + sqle.getLocalizedMessage());
            sqle.printStackTrace();
        }
        
    }
    
    public static  void main(String argv[]) {
        //Install a = new Install();
        //a.destroy();
        
        /*	try{
            Analysis a = new Analysis();
            Analysis.distOrderBy(Analysis.getConnection(), argv[0]);
            //Analysis.distGroupBy(Analysis.getConnection(), argv[0]);
        }catch(Exception e){
            e.printStackTrace(System.out);
            }*/
    }
    
    public void finalize() throws java.lang.Throwable {
        destroy();
        super.finalize();
    }
}
